/*
	ShimakazeDet.cpp
	By Shimatsukaze
	Distributed under the terms of the MIT License.
	http://opensource.org/licenses/MIT
*/
	

#include <cstdio>
#include <algorithm>
#include <cmath>
#include <vector>
#include <cassert>
#define lowbit(x) ((x)&(-(x)))
#define sqr(x) ((x)*(x))
#include "ShimakazeDet.h"
#include "BeatDetektor.cpp"
using namespace std;
const float pi=acos(-1);
const float eps=0.1;
const float ignore=0.2;

struct Complex{
	float x, y;
	Complex (float x=0, float y=0):x(x), y(y){}
	Complex operator + (const Complex &an) const
	{
		return Complex(x+an.x,y+an.y);
	}
	Complex operator - (const Complex &an) const
	{
		return Complex(x-an.x,y-an.y);
	}
	Complex operator * (const Complex &an) const
	{
		return Complex(x*an.x-y*an.y,x*an.y+y*an.x);
	}
	Complex operator / (const float &an) const
	{
		return Complex(x/an,y/an);
	}
};
vector<Complex> sz;
int N;
vector<float> sound;
vector<float> beat;
vector<float> energy;
BeatDetektor BR=BeatDetektor(85,169);

static void FFT(vector<Complex> &x, int lbit, int type)
{
	int l=1<<lbit;
	for (int i=0; i<l; i++)
	{
		int cur=0;
		for (int j=0; j<lbit; j++)
			cur=(cur<<1)|((i>>j)&1);
		if (cur>i)
		{
			swap(x[cur],x[i]);
		}
	}
	for (int h=1; h<l; h<<=1)
	{
		Complex w0=Complex(cos(pi/h),sin(-pi*type/h));
		for (int i=0; i<l; i+=h<<1)
		{
			Complex w=Complex(1,0);
			for (int j=i; j<i+h; j++)
			{
				Complex f=x[j], t=x[j+h]*w;
				x[j]=f+t;
				x[j+h]=f-t;
				w=w*w0;
			}
		}
	}
	if (type==-1)
	{
		for (int i=0; i<l; i++)
			x[i]=x[i]/l;
	}
}

static int setSampleCount(lua_State *L)
{
	N=lua_tonumber(L,1);
	if (N==-1)
	{
		BR.reset();
		beat.clear();
		energy.clear();
	}
	sound.clear();
	return 0;
}

static int getSample(lua_State *L)
{
	float val=lua_tonumber(L,1);
	sound.push_back(val);
	return 0;
}

static int calculate(lua_State *L)
{
	float curTime=lua_tonumber(L,1);
	int size=sound.size();
	if (lowbit(size)!=size)
		return 0;
	printf("%fs: Begin FFT: size %d\n",curTime,size);
	int cnt=0;
	for (int i=size>>1; i; i>>=1)
		cnt++;
	sz.clear();
	for (int i=0; i<size; i++)
		sz.push_back(Complex(sound[i],0));
	FFT(sz,cnt,1);
	for (int i=0; i<size; i++)
		sound[i]=sqrt(sqr(sz[i].x)+sqr(sz[i].y));
	BR.process(curTime,sound);
	if (BR.isKick())
	{
		printf("Beat At %f\n",curTime);
		float sum=0;
		for (int i=0; i<size; i++)
			sum+=sound[i];
		sum/=size;
		beat.push_back(curTime);
		energy.push_back(sum);
	}
	printf("Current winning BPM: %f\n",60.0/BR.winning_bpm_lo);
	return 0;
}

static int getFFT(lua_State *L)
{
	int size=sound.size();
	if (lowbit(size)!=size)
		return 0;
	int cnt=0;
	for (int i=size>>1; i; i>>=1)
		cnt++;
	sz.clear();
	for (int i=0; i<size; i++)
		sz.push_back(Complex(sound[i],0));
	FFT(sz,cnt,1);
	for (int i=0; i<size/2; i++)
		lua_pushnumber(L,sqrt(sqr(sz[i].x)+sqr(sz[i].y))/size);
	return size/2;
}

static int getBPM(lua_State *L)
{
	float interval=BR.winning_bpm_lo;
	int best=-1, val=-1;
	for (int i=1; i<=100; i++)
	{
		int pos=rand()%beat.size(), cur=0;
		for (int j=0; j<beat.size(); j++)
		{
			float tmp=(beat[j]-beat[pos])/interval;
			if (fabs(tmp-(int)tmp)<=eps)
			{
				if (j<pos)
					pos=j;
				cur++;
			}
		}
		if (cur>val)
		{
			val=cur;
			best=pos;
		}
	}
	float back=beat[best];
	printf("Best at #%d, appear %d times.\n",best,val);
	lua_pushnumber(L,BR.winning_bpm_lo);
	lua_pushnumber(L,back);
	return 2;
}

static int getBeats(lua_State *L)
{
	int size=beat.size();
	float ave=0;
	for (int i=0; i<size; i++)
		ave+=energy[i];
	ave/=size;
	vector<float>::iterator i1, i2;
	for (i1=beat.begin(), i2=energy.begin(); i1!=beat.end();)
	{
		if ((*i2)<=ave*ignore)
		{
			printf("At position %f: energy=%f/%f, erased.\n",*i1,*i2,ave);
			i1=beat.erase(i1);
			i2=energy.erase(i2);
		}
		else
		{
			i1++;
			i2++;
		}
	}
	for (int i=0; i<beat.size(); i++)
		lua_pushnumber(L,beat[i]);
	return beat.size();
}

static const struct luaL_Reg myLib[] = 
{
	{"init",setSampleCount},
	{"insert",getSample},
	{"calculate",calculate},
	{"FFT",getFFT},
	{"BPM",getBPM},
	{"beats",getBeats},
	{NULL, NULL}
};

extern "C" int luaopen_ShimakazeDet(lua_State *L) 
{
	freopen("ShimakazeDet.log","w",stdout);
	luaL_register(L, "ShimakazeDet", myLib);
	return 1;
}
