require("SDetector")

function playingDraw()
	if value==nil then
		analyzeDraw()
		return
	end
	love.graphics.setColor(255,255,255,255)
	love.graphics.setLineWidth(1)
	love.graphics.print("Current: "..value,10,10)
	love.graphics.print("Process: "..source:tell()*SD.rate/SD.count,10,40)
	if #(back)==0 then
		return
	end
	if signal then
		bgcolor=128
	else
		bgcolor=math.max(0,bgcolor-8)
	end
	love.graphics.setColor(255,255,255,bgcolor)
	love.graphics.rectangle("fill",0,0,width,height)
	love.graphics.setColor(255,255,255,255)

	for i=1,width-1 do
		local pos=i*SD.count/width
		local unit=SD.count/width
		local val=0
		for j=math.ceil(pos-unit/2),math.floor(pos+unit/2) do
			val=val+math.abs(SD:getSample(j))
		end
		val=val/(1-math.ceil(pos-unit)+math.floor(pos+unit))
		love.graphics.line(i,(height-val*height)/2,i,(height+val*height)/2)
	end
	if (signal) then
		love.graphics.setColor(255,0,0,255)
	else
		love.graphics.setColor(255,255,0,255)
	end
	love.graphics.line(playPos,0,playPos,height)

	local size=width/#(back)
	love.graphics.setLineWidth(size)
	if signal then
		love.graphics.setColor(255,0,0,255)
	else
		love.graphics.setColor(0,255,255,255)
	end

	for i=1,#(back) do
		local tmp=(i-0.5)*size
		love.graphics.line(tmp,height,tmp,height*(1-back[i]))
	end
end

function musicStop()
	love.graphics.setColor(255,255,255,255)
	love.graphics.print("Please drop music to load.",10,10)
end

function musicAnalyze(dt)
	local beginT=love.timer.getTime()
	while love.timer.getTime()-beginT<0.02 do
		process=SD:analyze(pos)
		if process==-1 then
			SD:finish()
			pos=1
			love.update=playingUpdate
			love.draw=playingDraw
			source:play()
			return
		end
		pos=pos+1
	end
end

function analyzeDraw()
	love.graphics.setColor(255,255,255,255)
	love.graphics.print(string.format("Analyzing: %5.2f%% Completed.",process*100),10,10)
end

function love.filedropped(file)
	if source then
		source:stop()
	end
	love.window.setTitle("Loading music file...")
	music=love.sound.newSoundData(file)
	source=love.audio.newSource(music)
	SD:loadData(music)
	love.update=musicAnalyze
	love.draw=analyzeDraw
	pos=1
	process=0
	love.window.setTitle("Playing - "..file:getFilename())
end

function love.load()
	math.randomseed(os.time())
	width=love.graphics.getWidth()
	height=love.graphics.getHeight()
	love.graphics.setFont(love.graphics.newFont(20))
	bgcolor=0
	love.draw=musicStop
end

function playingUpdate(dt)
	collectgarbage()
	if source:isStopped() then
		love.update=nil
		love.draw=musicStop
		return
	end
	local tSec=source:tell()
	local t=math.ceil(tSec*SD.rate)
	value=SD:getSample(t)
	playPos=t*width/SD.count

	signal=false
	while pos<=#(SD.beat) and tSec>=SD.beat[pos] do
		signal=true
		pos=pos+1
	end
	back={SD:getFFT(t)}
end